#!/usr/bin/python

import dbus
import dbus.service
import dbus.mainloop.glib
import gobject
import slip.dbus.service
from slip.dbus import polkit
import os
import selinux
from subprocess import Popen, PIPE, STDOUT

class selinux_server(slip.dbus.service.Object):
    default_polkit_auth_required = "org.fedoraproject.selinux.semanage"
    def __init__ (self, *p, **k):
        super(selinux_server, self).__init__(*p, **k)

    #
    # The semanage method runs a transaction on a series of semanage commands,
    # these commnds can take the output of customized
    # 
    @dbus.service.method("org.fedoraproject.selinux", in_signature='s')
    def semanage(self, buf):
        fd = open("/run/selinux.input", "w")
        fd.write(buf)
        fd.close()
        p = Popen(["/usr/sbin/semanage", "-i", "/run/selinux.input"],stdout=PIPE, stderr=PIPE)
        output = p.communicate()
        if p.returncode and p.returncode != 0:
            raise OSError("Failed update SELinux configuration: %s", output)
        
    #
    # The customized method will return all of the custommizations for policy
    # on the server.  This output can be used with the semanage method on 
    # another server to make the two systems have duplicate policy.
    #
    @dbus.service.method("org.fedoraproject.selinux", in_signature='', out_signature='s')
    def customized(self):
        p = Popen(["/usr/sbin/semanage", "-o", "/run/selinux.output"],stdout=PIPE, stderr=PIPE)
        output = p.communicate()
        if p.returncode and p.returncode != 0:
            raise OSError("Failed update SELinux configuration: %s", output)
        fd = open("/run/selinux.output", "r")
        buf = fd.read()
        fd.close()
        return buf
        
    #
    # The restorecon method modifies any file path to the default system label
    #
    @dbus.service.method("org.fedoraproject.selinux", in_signature='s')
    def restorecon(self, path):
        selinux.restorecon(str(path))

    #
    # The setenforce method turns off the current enforcement of SELinux
    #
    @dbus.service.method("org.fedoraproject.selinux", in_signature='i')
    def setenforce(self, value):
        selinux.security_setenforce(value)

if __name__ == "__main__":
        mainloop = gobject.MainLoop()
        dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
        system_bus = dbus.SystemBus()
        name = dbus.service.BusName("org.fedoraproject.selinux", system_bus)
        object = selinux_server(system_bus, "/org/fedoraproject/selinux/object")
        slip.dbus.service.set_mainloop(mainloop)
        mainloop.run()
